<?php
/**
 * A generic controller class. You should extend this base class if you want to
 * write your own controller. You get the advantage that you can use the pre and
 * post filters.
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2011 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class BaseController extends BaseFrameworkSystem implements Registerable {
	/**
	 * Generic filter chains
	 */
	private $filterChains = array();

	// Exception constants
	const EXCEPTION_FILTER_CHAIN_INVALID = 0xf10;

	/**
	 * Protected constructor
	 *
	 * @param	$className	Name of the class
	 * @return	void
	 */
	protected function __construct ($className) {
		// Call parent constructor
		parent::__construct($className);

		// Initialize both filter chains
		$this->initFilterChain('pre');
		$this->initFilterChain('post');

		// Add this controller to the registry
		Registry::getRegistry()->addInstance('controller', $this);
	}

	/**
	 * Private method to initialize a given filter chain
	 *
	 * @param	$filterChain	Name of the filter chain
	 * @return	void
	 */
	protected function initFilterChain ($filterChain) {
		//* DEBUG: */ $this->debugOutput('CONTROLLER: ' . $filterChain . ' init: START');
		$this->filterChains[$filterChain] = ObjectFactory::createObjectByConfiguredName('filter_chain_class');
		//* DEBUG: */ $this->debugOutput('CONTROLLER: ' . $filterChain . ' init: FINISHED');
	}

	/**
	 * Adds a filter to a given filter chain
	 *
	 * @param	$filterChain	Chain of the filter
	 * @param	$filterInstance		An instance of a filter
	 * @return	void
	 * @throws	InvalidFilterChainException	If the filter chain is invalid
	 */
	protected function addFilter ($filterChain, Filterable $filterInstance) {
		//* DEBUG: */ $this->debugOutput('CONTROLLER: ' . $filterChain . ',' . $filterInstance->__toString(). ' add: START');

		// Test if the filter is there
		if (!isset($this->filterChains[$filterChain])) {
			// Throw an exception here
			throw new InvalidFilterChainException(array($this, $filterChain), self::EXCEPTION_FILTER_CHAIN_INVALID);
		} // END - if

		// Add the filter
		$this->filterChains[$filterChain]->addFilter($filterInstance);
		//* DEBUG: */ $this->debugOutput('CONTROLLER: ' . $filterChain . ',' . $filterInstance->__toString(). ' add: FINISH');
	}

	/**
	 * Adds a filter to the pre filter chain
	 *
	 * @param	$filterInstance		An instance of a filter
	 * @return	void
	 */
	public function addPreFilter (Filterable $filterInstance) {
		// Add the pre filter
		$this->addFilter('pre', $filterInstance);
	}

	/**
	 * Adds a filter to the post filter chain
	 *
	 * @param	$filterInstance		An instance of a filter
	 * @return	void
	 */
	public function addPostFilter (Filterable $filterInstance) {
		// Add the post filter
		$this->addFilter('post', $filterInstance);
	}

	/**
	 * Executes given filter chain chain
	 *
	 * @param	$filterChain		Chain of the filter to execute
	 * @param	$requestInstance	An instance of a request class
	 * @param	$responseInstance	An instance of a response class
	 * @return	void
	 * @throws	InvalidFilterChainException	If the filter chain is invalid
	 */
	protected function executeFilters ($filterChain, Requestable $requestInstance, Responseable $responseInstance) {
		// Test if the filter is there
		if (!isset($this->filterChains[$filterChain])) {
			// Throw an exception here
			throw new InvalidFilterChainException(array($this, $filterChain), self::EXCEPTION_FILTER_CHAIN_INVALID);
		} // END - if

		// Run all filters
		$this->filterChains[$filterChain]->processFilters($requestInstance, $responseInstance);
	}

	/**
	 * Executes all pre filters
	 *
	 * @param	$requestInstance	An instance of a request class
	 * @param	$responseInstance	An instance of a response class
	 * @return	void
	 */
	protected function executePreFilters (Requestable $requestInstance, Responseable $responseInstance) {
		// Execute all pre filters
		$this->executeFilters('pre', $requestInstance, $responseInstance);
	}

	/**
	 * Executes all post filters
	 *
	 * @param	$requestInstance	An instance of a request class
	 * @param	$responseInstance	An instance of a response class
	 * @return	void
	 */
	protected function executePostFilters (Requestable $requestInstance, Responseable $responseInstance) {
		// Execute all post filters
		$this->executeFilters('post', $requestInstance, $responseInstance);
	}
}

// [EOF]
?>
